import datetime
import jwt
import time
from peewee import SQL
from libraries.system.manager import AbstractManager
from libraries.system.model import BaseModel
from libraries.system.crypt import Crypt
from my_package_profile_manager.libraries.user.model import UserModel
from my_package_profile_manager.libraries.profile.model import UserProfileModel
from my_package_profile_manager.libraries.api.model import ApiKeyModel, ApiTokenModel
from application.settings import application_settings
from libraries.system.logger import Log


class AuthManager(AbstractManager):
    @BaseModel._meta.database.atomic()
    def user_login(self, **data: dict) -> dict:
        self.response = {
            "status": 401,
            "type": "incorrect",
            "authenticated": False,
            "message": "Invalid email or password.",
            "success_status": False
        }
        try:
            user = UserModel.get_or_none(UserModel.email == data['email'])
            if user is not None:
                crypt = Crypt()
                decrypted_password = crypt.decrypt(user.password)
                if data["password"] == decrypted_password:
                    if user.status == "inactive":
                        self.response["type"] = user.status
                        self.response["message"] = "Activation Pending. Please check your inbox for activation email."
                    elif user.status == "suspended":
                        self.response["message"] = "Account suspended. Please contact administrator."
                        self.response["type"] = user.status
                    else:
                        payload: dict = {}
                        payload_user_details = {
                            "id": user.id,
                            "name": user.name,
                            "email": user.email,
                            "status": user.status,
                            'preference': self.get_preference(user.id),
                            'image': self.get_user_image(user.id),
                            'created_at': str(user.created_at),
                            'updated_at': str(user.updated_at),
                            'login_time': str(datetime.datetime.now())
                        }

                        key_details = self.get_key_details()
                        if key_details is None:
                            self.response["message"] = 'Please generate api key and token'
                        # encrypted_user_details = crypt.encrypt(json_encode(user_details))
                        """ Generate authentication token """
                        expiry_time = None
                        if key_details['expiry_date'] is not None:
                            expiry_time = time.mktime(
                                key_details['expiry_date'].timetuple())
                        payload["payload"] = payload_user_details
                        if expiry_time is not None:
                            payload["exp"] = expiry_time
                        jwtToken = jwt.encode(
                            payload,
                            key=application_settings["jwt"]["secret"],
                            algorithm=application_settings["jwt"]["algorithm"]
                        )
                        self.response["type"] = user.status
                        self.response["token"] = jwtToken.decode()
                        self.response["authenticated"] = True
                        self.response['success_status'] = True
                        self.response["detail"] = payload_user_details
                        self.response["detail"]['token'] = jwtToken.decode()
                        self.response["message"] = "Logged in successfully."
                        self.response['status'] = 200
                        self.insert_token_to_db(
                            payload_user_details, self.response["token"], key_details['id'])

        except Exception as e:
            Log().entry(e)  # loggin error
        finally:
            return self.response

    @BaseModel._meta.database.atomic()
    def insert_token_to_db(slef, user_detail, token, key_id):
        # insert user token to db after successfull authentication
        try:
            api_token_model = ApiTokenModel()
            api_token_model.user_id = user_detail["id"]
            api_token_model.api_key_id = key_id
            api_token_model.token = token
            api_token_model.revoked = False
            api_token_model.save()
        except Exception as e:
            Log().entry(e)  # loggin error

    @BaseModel._meta.database.atomic()
    def get_key_details(self):
        # returns expiry date of token and secret. encryption key was used from settings for now
        try:
            name = (list(application_settings['services'].keys())[0])
            key = application_settings["services"][name]["api_key"]
            secret = application_settings["services"][name]["api_token"]
            key_detail = ApiKeyModel.get_or_none(SQL("key = %s and secret = %s", [key, secret]))
            if key_detail is not None:
                return {
                    'id': key_detail.id,
                    'expiry_date': key_detail.expiry_date
                }
            return None
        except Exception as e:
            Log().entry(e)  # loggin error

    @BaseModel._meta.database.atomic()
    def get_preference(self, user_id=None):
        response: dict = {"view": "list", "menu_open": True}
        try:
            user_detail = UserProfileModel.get_or_none(UserProfileModel.user_id == user_id)
            if user_detail is not None:
                menu_open = user_detail.preferences['menu_open'] if 'menu_open' in user_detail.preferences else True
                view = user_detail.preferences["view"] if 'view' in user_detail.preferences else 'list'
                response = {"view": view, "menu_open": menu_open}
        except Exception as e:
            Log().entry(e)
        finally:
            return response

    @BaseModel._meta.database.atomic()
    def get_user_image(self, user_id):
        try:
            # get image of user
            user_profile = UserProfileModel.get_or_none(UserProfileModel.user_id == user_id)
            if user_profile is not None:
                return user_profile.image
            return
        except Exception as e:
            Log().entry(e)  # loggin error