import random, string, datetime
from application.settings import application_settings
from libraries.system.queue import RedisQueue
from libraries.system.email import Email
from libraries.system.manager import AbstractManager
from libraries.system.model import BaseModel
from libraries.system.crypt import Crypt
from my_package_profile_manager.libraries.user.model import UserModel
from my_package_profile_manager.libraries.profile.model import UserProfileModel
from my_package_profile_manager.libraries.email_templates.model import EmailTemplateModel
from libraries.system.logger import Log
from peewee import SQL

"""
    User manager 
"""
class UserManager(AbstractManager):
    def __init__(self):
        self.response: dict = dict()
        self.response['success_status'] = False

    @BaseModel._meta.database.atomic()
    def user_registration(self, **data: dict) -> dict:
        # user registration
        self.response = {
            "status": 401,
            "message": "Could not register user",
            "success_status": False
        }
        try:
            # password hashing
            crypt = Crypt()
            data['password'] = crypt.encrypt(data['password'])
            data['activation_code'] = self.random_string(35)
            data['email'] = data['email'].lower()
            data['created_at'] = datetime.datetime.now()
            data['status'] = 'inactive'
            user = UserModel(**data)
            if user.save():
                # save to user profile
                # user_profile = UserProfileModel()
                # user_profile.user_id = user.id
                # user_profile.created_at = datetime.datetime.now()
                # user_profile.save()
                self.response["detail"] = {
                    "id": user.id,
                    "name": user.name,
                    "email": user.email}
                self.response["message"] = "Registration successful.Please check your email inbox to activate your account."
                self.response['success_status'] = True
                self.response['status'] = 200
                self.response['activation_code'] = user.activation_code
                RedisQueue('email').enqueue(self.notify_registered_user(
                                            user.email,
                                            user.name,
                                            user.activation_code
                                            ))
        except Exception as e:
            Log().entry(e)
        finally:
            return self.response

    @BaseModel._meta.database.atomic()
    def account_verification(self, **data: dict) -> dict:
        # account_verification
        self.response["message"] = "Email verification failed."
        self.response['status'] = 401
        try:
            # get user by activation code
            user = UserModel.get_or_none(UserModel.email == data["email"])
            if user is not None:
                user.status = "active"
                user.email_verified = True
                user.activation_code = None
                self.response['message'] = "Email verified."
                self.response['success_status'] = True
                self.response['status'] = 200
                if user.save():
                    self.response["detail"] = {
                        "id": user.id,
                        "name": user.name,
                        "email": user.email}
        except Exception as e:
            Log().entry(e)
        finally:
            return self.response


    @BaseModel._meta.database.atomic()
    def forget_password(self, email: str) -> dict:
        self.response["message"] = "Email does not exist in our system."
        self.response['status'] = 401
        try:
            user = UserModel.get_or_none(UserModel.email == email)
            if user is not None:
                self.response['detail']={}
                self.response['success_status'] = True
                self.response["message"] = "Email match."
                self.response['status'] = 200
                user.activation_code = self.random_string(35)
                RedisQueue('email').enqueue(self.notify_forget_password(
                    user.email,
                    user.name,
                    user.activation_code
                ))
        except Exception as e:
            Log().entry(e)
        finally:
            return self.response

    @BaseModel._meta.database.atomic()
    def update_profile(self, **data: dict) -> dict:
        self.response["update_profile"] = False
        self.response['message'] = "User profile could not update."
        try:
            user = UserModel.get_or_none(UserModel.id == data['user_id'])
            if user is not None:
                profiles = UserProfileModel.get_or_none(UserProfileModel.user_id == user.id)
                if profiles is not None:
                    profiles.social_media = {
                        "facebook": data["facebook"],
                        "twitter": data["twitter"],
                        "linkedin": data["linkedin"]
                        }
                    profiles.user_id = user.id
                    profiles.permanent_address = data["permanent_address"]
                    profiles.temporary_address = data["temporary_address"]
                    profiles.phone = data["phone"]
                    profiles.mobile = data["mobile"]
                    profiles.alternate_email = data["alternate_email"]
                    profiles.updated_at = datetime.datetime.now()
                    if profiles.save():
                        self.response["success_status"] = True
                        self.response["detail"] = {
                            "permanent_address": profiles.permanent_address,
                            "temporary_address": profiles.temporary_address,
                            "phone": profiles.phone,
                            "mobile": profiles.mobile,
                            "alternate_email": profiles.alternate_email,
                            "social_media": profiles.social_media
                        }
                        self.response['message'] = "User profile updated."

        except Exception as e:
            Log().entry(e)
        finally:
            return self.response

    @BaseModel._meta.database.atomic()
    def notify_registered_user(self, to, full_name, activation_code):
        """ Notify register user via email. """
        try:
            app_name = application_settings["application_name"]
            subject = 'Welcome to ' + app_name
            year = datetime.date.today().year
            name = (list(application_settings['services'].keys())[0])
            web_url = application_settings["services"][name]["web_url"]
            email_template_detail = EmailTemplateModel.select(EmailTemplateModel.content).where(
                                    EmailTemplateModel.type == 'sign_up',
                                    EmailTemplateModel.status=='active'
                                ).order_by(EmailTemplateModel.created_at.desc()).get()
            if email_template_detail is not None:
                # make body of email
                body = email_template_detail.content.replace(
                    "{{full_name}}", full_name
                ).replace(
                    "{{year}}", str(year)
                ).replace(
                    "{{application_name}}", app_name
                ).replace(
                    "{{activation_code}}", activation_code
                ).replace(
                    "{{web_url}}", web_url
                )

            email = Email()
            email.send(to, subject, body)

        except Exception as e:
            Log().entry(e)  # loggin error

    @BaseModel._meta.database.atomic()
    def notify_forget_password(self,to,full_name,user_email):
        try:
            app_name = application_settings["application_name"]
            subject = 'Welcome to ' + app_name
            year = datetime.date.today().year
            name = (list(application_settings['services'].keys())[0])
            web_url = application_settings["services"][name]["web_url"]
            email_template_detail = EmailTemplateModel.select(EmailTemplateModel.content).where(
                                    EmailTemplateModel.type == 'forget_password',
                                    EmailTemplateModel.status=='active'
                                ).order_by(EmailTemplateModel.created_at.desc()).get()
            if email_template_detail is not None:
                # make body of email
                body = email_template_detail.content.replace(
                    "{{full_name}}", full_name
                ).replace(
                    "{{year}}", str(year)
                ).replace(
                    "{{application_name}}", app_name
                ).replace(
                    "{{web_url}}", web_url
                )

            email = Email()
            email.send(to, subject, body)
        except Exception as e:
            Log().entry(e)  # loggin error


    @BaseModel._meta.database.atomic()
    def is_unique_email(self, email: str, id: int=None) -> bool:
        try:
            is_unique_email = UserModel.get_or_none(SQL("email = %s and id != %s", [email, id]))
            return is_unique_email is None
            # if is_unique_email is not None:
            #     return False
            # return True
        except Exception as e:
            Log().entry(e)


    def random_string(self, length: int) -> str:
        """Generate a random string of x length """
        random_list = []
        for i in range(length):
            random_list.append(random.choice(string.ascii_uppercase + string.ascii_lowercase + string.digits))
        return ''.join(random_list)

